package com.caimei.service.order.impl;

import com.alibaba.fastjson.JSONArray;
import com.caimei.entity.*;
import com.caimei.mapper.order.CmShopOrderMapper;
import com.caimei.mapper.order.OrderMapper;
import com.caimei.mapper.order.OrderProductMapper;
import com.caimei.mapper.products.OrganizeProductsMapper;
import com.caimei.mapper.user.LoginMapper;
import com.caimei.mapper.user.PersonalMapper;
import com.caimei.mapper.user.UserMapper;
import com.caimei.service.order.OrderService;
import com.caimei.utils.AppUtils;
import com.caimei.utils.MathUtil;
import com.caimei.utils.NoUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class OrderServiceImpl implements OrderService {
    protected static final Logger logger = LoggerFactory.getLogger(OrderServiceImpl.class);

    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private PersonalMapper personalMapper;
    @Autowired
    private LoginMapper loginMapper;
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private OrganizeProductsMapper organizeProductsMapper;
    @Autowired
    private OrderProductMapper orderProductMapper;
    @Autowired
    private CmShopOrderMapper cmShopOrderMapper;

    @Value("${miniprogram.domain}")
    private String domain;

    @Override
    public Map<String, Object> confirmOrder(Integer userId, String classifyIDS, Integer organizeID, String productIDs, Integer count) {
        Map<String, Object> map = new HashMap();
        //商品信息
        List<String> list = new ArrayList();
        if (productIDs.contains(",")) {
            String[] split = productIDs.split(",");
            list = Arrays.asList(split);
        } else {
            list.add(productIDs);
        }
        List<Shop> shopList = orderMapper.findShop(list);
        for (Shop shop : shopList) {
            List<CmOrganizeProducts> productsList = new ArrayList();
            Double totalPrice = 0d;
            if (count == null) {
                //购物车购买
                Map<String, Object> params = new HashMap();
                params.put("productIDs", list);
                params.put("shopID", shop.getShopID());
                params.put("userId", userId);
                productsList = orderMapper.findAllCommodity(params);
                for (CmOrganizeProducts products : productsList) {
                    products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
                    totalPrice += products.getRetailPrice() * products.getProductCount();
                }
            } else {
                //立即购买
                CmOrganizeProducts products = orderMapper.findCommodity(productIDs, shop.getShopID());
                products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
                products.setProductCount(count);
                totalPrice = products.getRetailPrice() * products.getProductCount();
                productsList.add(products);
            }
            shop.setLogo(AppUtils.getImageURL("shopLogo", shop.getLogo(), 0, domain));
            shop.setTotalPrice(totalPrice);
            shop.setProductsList(productsList);
        }
        //发票信息
        UserInvoiceInfo invoice = personalMapper.findInvoice(userId);
        //运费
        FreePostageCard freePostageCard = orderMapper.queryFree(userId);
        String freight = "";
        //-1到付，0包邮,-2仪器到付其它包邮
        boolean classify = true;//判断是否全部是仪器
        for (String classifyID : classifyIDS.split(",")) {
            if (classifyID != null && classifyID.equals("3") && classify) {
                classify = true;
            } else {
                classify = false;
            }
        }
        if (freePostageCard == null || classify) {
            freight = "-1";
        } else if (classifyIDS.contains("3")) {
            freight = "-2";
        } else {
            freight = "0";
        }
        //客服热线号码
        String cellPhone = loginMapper.cellPhone(organizeID);
        //可用余额
        Double userMoney = personalMapper.ableUserMoney(userId);
        if (userMoney == null) userMoney = 0d;
        map.put("shopList", shopList);
        map.put("invoice", invoice);
        map.put("freight", freight);
        map.put("cellPhone", cellPhone);
        map.put("userMoney", userMoney);
        return map;
    }

    @Override
    public String shareOrder(Integer orderID) {
        OrderShareCode orderShareCode = orderMapper.queryCode(orderID);
        //生成分享码
        String shareCode = RandomStringUtils.random(4, false, true);
        //过期时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DATE, 1);
        if (orderShareCode != null && orderShareCode.getExpiredTime().compareTo(new Date()) == 1) {
            return orderShareCode.getShareCode();
        } else if (orderShareCode != null) {
            orderShareCode.setShareCode(shareCode);
            orderShareCode.setExpiredTime(calendar.getTime());
            orderShareCode.setDelFlag("0");
            orderMapper.updateCode(orderShareCode);
            return String.valueOf(shareCode);
        } else {
            OrderShareCode code = new OrderShareCode();
            code.setOrderID(orderID);
            code.setShareCode(shareCode);
            code.setExpiredTime(calendar.getTime());
            code.setAddTime(new Date());
            code.setDelFlag("0");
            orderMapper.insertCode(code);
            return String.valueOf(shareCode);
        }
    }

    @Override
    public WxJsonModel verifyShareCode(String shareCode, String openid, Integer orderID, Integer userID,
                                       Integer organizeID) {
        WxJsonModel model = WxJsonModel.newInstance();
        CmOperationUser operationUser = loginMapper.doLogin(openid, organizeID);
        if (operationUser != null && operationUser.getUserID() == userID) {
            return model.success();
        }
        CmMallShareCodeRecord shareCodeRecord = orderMapper.findShareCodeRecord(openid, orderID);
        if (shareCodeRecord != null && shareCodeRecord.getExpiredTime().compareTo(new Date()) == 1) {
            return model.success();
        }
        OrderShareCode orderShareCode = orderMapper.queryCode(orderID);
        if (orderShareCode.getShareCode().equals(shareCode)) {
            CmMallShareCodeRecord codeRecord = new CmMallShareCodeRecord();
            codeRecord.setOpenid(openid);
            codeRecord.setAddTime(new Date());
            codeRecord.setOrderID(orderShareCode.getOrderID());
            codeRecord.setDelFlag("0");
            codeRecord.setShareCodeId(orderShareCode.getId());
            orderMapper.saveShareCodeRecord(codeRecord);
            return model.success();
        } else if (orderShareCode.getExpiredTime().compareTo(new Date()) == -1) {
            return model.error("分享码失效");
        } else {
            return model.error("分享码错误");
        }
    }

    @Override
    public Map<String, Object> particulars(Integer orderID) {
        Map<String, Object> map = new HashMap();
        //收货地址
        BpOrderUserInfo userInfo = orderMapper.findUserInfo(orderID);
        //订单信息,运费
        CmOrder order = orderMapper.findOrder(orderID);
        order.setOrderMark("#" + order.getOrderID() + "#");
        //商品信息
        String[] ShopOrderIDs = {};
        if (order.getShopOrderIDs().contains(",")) {
            ShopOrderIDs = order.getShopOrderIDs().split(",");
        } else {
            ShopOrderIDs = new String[]{order.getShopOrderIDs()};
        }
        List<CmShopOrder> shopOrderList = orderMapper.findAllShopOrder(ShopOrderIDs);
        for (CmShopOrder shopOrder : shopOrderList) {
            List<CmOrderProduct> orderProductList = orderMapper.findOrderProduct(shopOrder.getShopOrderID());
            for (CmOrderProduct orderProduct : orderProductList) {
                orderProduct.setProductImage(AppUtils.getImageURL("product", orderProduct.getProductImage(), 0, domain));
            }
            shopOrder.setCmOrderProducts(orderProductList);
            shopOrder.setShopLogo(AppUtils.getImageURL("shopLogo", shopOrder.getShopLogo(), 0, domain));
        }
        //发票信息
        UserInvoiceInfo invoice = personalMapper.findInvoice(order.getUserID());
        //支付记录
        List<CmDiscernReceipt> discernReceiptList = orderMapper.findDiscernReceipt(orderID);
        //退款记录
        List<CmReturnedPurchase> returnedPurchaseList = orderMapper.findReturnedPurchase(order.getOrderID());
        //客服热线号码
        String cellPhone = loginMapper.cellPhone(orderID);
        //分享码
        String shareCode = orderMapper.findShareCode(orderID);
        map.put("order", order);
        map.put("userInfo", userInfo);
        map.put("shopOrderList", shopOrderList);
        map.put("invoice", invoice);
        map.put("discernReceiptList", discernReceiptList);
        map.put("returnedPurchaseList", returnedPurchaseList);
        map.put("cellPhone", cellPhone);
        map.put("shareCode", shareCode);
        return map;
    }

    @Override
    public WxJsonModel saveOrderInfo(WxJsonModel wxJsonModel, Integer userId, Integer organizeID, String cartType, Integer addressID, Object orderInfo, String balanceDeductionFlag, Double orderShouldPayFee, Integer cmOperationID) {
        /**获取用户信息*/
        User user = userMapper.findUserById(userId);
        /**获取用户包邮卡*/
        FreePostageCard freePostageCard = orderMapper.queryFree(userId);
        if (null == user) {
            return wxJsonModel.error("1000", "用户账户异常");
        }
        Integer clubID = user.getClubID();
        boolean isIncludedInstruments = false;//是否包含仪器(默认不包含仪器)
        boolean isIncludedProduct = false;//是否包含商品(默认不包含仪器)
        CmOrder cmOrder = new CmOrder();
        List<CmOrderProduct> orderProductList = new ArrayList<>();//整理订单商品列表
//        List<Integer> shopIDs = new ArrayList<>();//整理子订单列表
        HashMap<Integer, String> shopIDs = new HashMap<>();
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String dateStr = sdf.format(date);

        /**订单信息转化*/
        List<Map<String, Object>> orderInfoList = (List<Map<String, Object>>) orderInfo;
        if (null != orderInfoList) {
            Integer productCount = 0;//统计购买所有商品的数量
            double productTotalFee = 0d;//商品总金额 （商品单价乘以数量，再加上税费[默认0]）
            //double orderTotalFee = 0d;//小计金额 （商品折后单价乘以数量，再加上税费[默认0]）商城下单和商品总金额一样
            double payTotalFee = 0d;//订单总额（小计金额减去经理折扣后，再加上运费[默认0])
            BigDecimal payableAmount = new BigDecimal(0);//真实支付金额（订单总额减去抵扣的账户余额）
            double balancePayFee = 0d;//余额支付金额
            for (Map<String, Object> shopOrderInfo : orderInfoList) {//遍历订单商品信息
                Integer shopId = (Integer) shopOrderInfo.get("shopId");
                String note = (String) shopOrderInfo.get("note");
                Object productInfo = shopOrderInfo.get("productInfo");//一个子订单对应的商品信息
                if (null == shopId) return wxJsonModel.error("1001", "供应商数据异常");
                if (null == productInfo) return wxJsonModel.error("1001", "订单商品数据异常");
//                shopIDs.add(shopId);
                shopIDs.put(shopId,note);

                //获取子订单商品信息
                List<Map<String, Object>> productInfoList = (List<Map<String, Object>>) productInfo;
                if (!CollectionUtils.isEmpty(productInfoList)) {
                    for (Map<String, Object> product : productInfoList) {//遍历所有商品
                        Integer productId = (Integer) product.get("productId");
                        Integer productNum = (Integer) product.get("productNum");
                        //统计商品总数量
                        productCount += productCount;
                        //获取商品信息
                        CmOrganizeProducts cmOrganizeProducts = organizeProductsMapper.selectProductById(productId);
                        if (null == cmOrganizeProducts) return wxJsonModel.error("1001", "订单商品数据异常");
                        if (null == productNum || productNum == 0) return wxJsonModel.error("1001", "商品购买数量异常");
                        Product cmProduct = organizeProductsMapper.selectCmProductById(cmOrganizeProducts.getProductID());
                        Double retailPrice = cmOrganizeProducts.getRetailPrice();//售价
                        Integer classifyID = cmOrganizeProducts.getClassifyID();
                        double productFee = retailPrice * productNum;//单个商品的金额
                        //统计商品总金额
                        productTotalFee += productFee;
                        if (classifyID == 3 && !isIncludedInstruments) {//判断是否存在仪器
                            isIncludedInstruments = true;
                        }
                        if (classifyID != 3 && !isIncludedProduct) {//判断是否存在商品
                            isIncludedProduct = true;
                        }

                        /**整理订单商品集合数据*/
                        CmOrderProduct cmOrderProduct = new CmOrderProduct();
                        cmOrderProduct.setShopID(cmOrganizeProducts.getShopID());
                        cmOrderProduct.setProductID(cmOrganizeProducts.getProductID());//此处保存采美库商品Id
                        cmOrderProduct.setOrganizeProductID(cmOrganizeProducts.getId());
                        cmOrderProduct.setOrganizeID(organizeID);
                        //预留在保存保存子订单的时候添加
//                        cmOrderProduct.setShopOrderID(0);
//                        cmOrderProduct.setShopOrderNo("");
                        cmOrderProduct.setProductNo(null);
                        cmOrderProduct.setNum(productNum);
                        cmOrderProduct.setPresentNum(0);
                        cmOrderProduct.setPrice(retailPrice);
                        cmOrderProduct.setProductUnit(cmProduct.getUnit());
                        cmOrderProduct.setNormalPrice(cmOrganizeProducts.getNormalPrice());
                        cmOrderProduct.setCostPrice(cmOrganizeProducts.getNormalPrice());
                        cmOrderProduct.setShopProductAmount(cmOrganizeProducts.getCostPrice() * productNum);//商品费=成本价快照*(购买数量  + 赠品数量)
                        cmOrderProduct.setPrice0(retailPrice);
                        cmOrderProduct.setPrice1(retailPrice);
                        cmOrderProduct.setTaxRate(0d);
                        cmOrderProduct.setAddedValueTax(0d);
                        cmOrderProduct.setTotalAddedValueTax(0d);
                        cmOrderProduct.setShopFee(cmOrganizeProducts.getCostPrice() * productNum);//应付供应商=成本价 * 商品数量
                        cmOrderProduct.setSingleShopFee(cmOrganizeProducts.getCostPrice());
                        cmOrderProduct.setOtherFee(0d);
                        cmOrderProduct.setSingleOtherFee(0d);
                        cmOrderProduct.setCmFee((cmOrganizeProducts.getRetailPrice() * productNum) - (cmOrganizeProducts.getCostPrice() * productNum));//应付采美=应付金额(应收金额)-应付供应商-应付第三方0-税费0）
                        cmOrderProduct.setSingleCmFee(cmOrganizeProducts.getRetailPrice() - cmOrganizeProducts.getCostPrice());
                        cmOrderProduct.setTotalAmount(cmOrganizeProducts.getRetailPrice() * productNum);
                        cmOrderProduct.setTotalFee(cmOrganizeProducts.getRetailPrice() * productNum);
                        cmOrderProduct.setTotalBeans(0d);
                        cmOrderProduct.setUseBalanceAmount(0d);
                        cmOrderProduct.setDiscountFee(0D);//经理折扣 优惠金额
                        cmOrderProduct.setPreferential(0d);//优惠金额
                        cmOrderProduct.setUseBalanceAmount(0d);
                        cmOrderProduct.setConfirmProductFlag("0");//订单商品供应商确认标志 0否 1是
                        cmOrderProduct.setDiscount(100d);
                        cmOrderProduct.setDiscountPrice(cmOrganizeProducts.getRetailPrice());
                        cmOrderProduct.setShopName(cmOrganizeProducts.getShopName());
                        cmOrderProduct.setName(cmProduct.getName());
                        cmOrderProduct.setPayStatus("0");
                        cmOrderProduct.setBuyAgainFlag("0");
                        cmOrderProduct.setNotOutStore(productNum); // 未出库数量
                        cmOrderProduct.setCommentFlag("0");//是否已评论 1 是 0 未评论
                        cmOrderProduct.setActPreferential(0d);
                        cmOrderProduct.setActType(null);
                        cmOrderProduct.setIsActProduct("0");//非活动商品
                        cmOrderProduct.setProductImage(AppUtils.getImageURL("product", cmProduct.getMainImage(), 0, domain));
                        orderProductList.add(cmOrderProduct);
                    }
                }
            }

            /**判断免邮标志*/
            if (isIncludedInstruments) {//包括仪器
                if (isIncludedProduct && null != freePostageCard) {//包含商品且有包邮卡
                    cmOrder.setFreePostFlag("-2");//免邮标志：-1到付，0包邮，1需要运费,-2仪器到付其它包邮【小程序只存在-1,0,-2】
                } else {//（1包含商品无包邮卡，2不包含商品）
                    cmOrder.setFreePostFlag("-1");
                }
            } else {//不包括仪器
                if (isIncludedProduct && null != freePostageCard) {//无仪器包含商品且有包邮卡
                    cmOrder.setFreePostFlag("0");//免邮标志：-1到付，0包邮，1需要运费,-2仪器到付其它包邮【小程序只存在-1,0,-2】
                } else {//（1无仪器包含商品无包邮卡）
                    cmOrder.setFreePostFlag("-1");
                }
            }

            /**统计订单应付总金额*/
            payTotalFee = productTotalFee;
            BigDecimal bdPayTotalFee = new BigDecimal(payTotalFee);
            cmOrder.setStatus(11);//待收款待发货(设置初始化状态)
            cmOrder.setReceiptStatus("1");//（收款买家）收款状态：1待收款、2部分收款、3已收款(设置初始化状态)
            cmOrder.setPayFlag("0");//是否已支付 未支付0 已支付1(设置初始化状态)

            /**判断余额使用*/
            boolean useBalanceFlag = false;//默认不适用余额
            if (StringUtils.equals("1", balanceDeductionFlag)) {//勾选余额抵扣
                //获取用户最新余额
                Double userMoney = personalMapper.ableUserMoney(userId);
                logger.info(">>>>>" + userId + "userMoney:" + userMoney);
                if (null != userMoney || 0 != userMoney) {
                    useBalanceFlag = true;
                    BigDecimal bdUserMoney = new BigDecimal(userMoney);
                    if (payTotalFee > userMoney) {//部分抵扣
                        balancePayFee = userMoney;
                        payableAmount = bdPayTotalFee.subtract(bdUserMoney);
                        user.setUserMoney(new BigDecimal(0));//余额抵扣用完
                        user.setAbleUserMoney(new BigDecimal(0));//余额抵扣用完
                        userMapper.updateUserMoney(user);
                        userMapper.updateAbleUserMoney(user);
                        cmOrder.setStatus(21);//部收款待发货
                        cmOrder.setReceiptStatus("2");
                    } else if (payTotalFee < userMoney) {//全部用余额抵扣，无需直接，直接变成支付完成
                        balancePayFee = payTotalFee;
                        user.setAbleUserMoney(bdUserMoney.subtract(bdPayTotalFee));
                        user.setUserMoney(bdUserMoney.subtract(bdPayTotalFee));
                        userMapper.updateAbleUserMoney(user);
                        userMapper.updateUserMoney(user);
                        cmOrder.setStatus(31);//已收款待发货
                        cmOrder.setReceiptStatus("3");
                        cmOrder.setPayFlag("1");
                    } else if (payTotalFee == userMoney) {//刚好抵扣完
                        balancePayFee = payTotalFee;
                        user.setAbleUserMoney(new BigDecimal(0));
                        user.setUserMoney(new BigDecimal(0));
                        userMapper.updateAbleUserMoney(user);
                        userMapper.updateUserMoney(user);
                        cmOrder.setStatus(31);//已收款待发货
                        cmOrder.setReceiptStatus("3");
                        cmOrder.setPayFlag("1");
                    }
                    cmOrder.setPayTime(dateStr);//支付时间

                }
            }

            /**判断发票使用*/
            UserInvoiceInfo info = personalMapper.findInvoice(userId);
            if (null != info) {
                cmOrder.setInvoiceFlag("2");//是否开发票 没开发票0 /开个人发票1 /开企业发票2【小程序目前只存在0,2】
            } else {
                cmOrder.setInvoiceFlag("0");
            }

            /**初始化主订单参数*/
            String orderNo = NoUtils.getCmOrderNo("X");
            cmOrder.setOrderNo(orderNo);
            cmOrder.setOrganizeID(organizeID);
            cmOrder.setUserID(userId);
            cmOrder.setBuyUserID(cmOperationID);//操作者的ID(下单人)
            cmOrder.setOrderSubmitType(2);// 2:员工帮会所下单
            cmOrder.setOrderType(1);//订单类型 协销订单0 、普通订单(自主订单)1
            cmOrder.setHasActProduct("0");
            cmOrder.setSendOutStatus("1");//待发货
            cmOrder.setProductTotalFee(productTotalFee);
            cmOrder.setOrderTotalFee(productTotalFee);
            cmOrder.setPayTotalFee(payTotalFee);
            cmOrder.setPayableAmount(payableAmount.doubleValue());
            cmOrder.setPreferential(0d);
            cmOrder.setDiscountFee(0d);//经理折扣
            cmOrder.setSpID(null);
            cmOrder.setMainSpID(null);
            cmOrder.setClubID(clubID);
            cmOrder.setOrderSource("6");//小程序
            cmOrder.setConfirmTime(dateStr);
            cmOrder.setOrderTime(dateStr);
            cmOrder.setProductCount(productCount);
            cmOrder.setPresentCount(0);
            cmOrder.setConfirmFlag("1");
            cmOrder.setClauseID(1);
            cmOrder.setClauseContent(null);
            cmOrder.setClauseName(null);
            cmOrder.setUpdateDate(dateStr);
            cmOrder.setDelFlag("0");
            cmOrder.setSplitFlag("0");//0不可拆分
            cmOrder.setPostageOrderFlag("0");//0不是邮费订单
            cmOrder.setBalancePayFee(balancePayFee);//余额支付金额
            cmOrder.setPayStatus("1");//(付款供应商)付款状态：1待付款、2部分付款、3已付款
            cmOrder.setOnlinePayFlag("0");//是否能走线上支付 0可以 1不可以 只能线下

            //保存住订单数据
            orderMapper.insert(cmOrder);
            Integer orderID = cmOrder.getOrderID();
            logger.info(">>>>>orderID："+orderID);

            if(useBalanceFlag){
                /**添加余额收支记录*/
                CmUserBalanceRecord cmUserBalanceRecord = new CmUserBalanceRecord();
                cmUserBalanceRecord.setUserId(userId);
                cmUserBalanceRecord.setType("2");
                cmUserBalanceRecord.setBalanceType("1");
                cmUserBalanceRecord.setAddDate(new Date());
                cmUserBalanceRecord.setAmount(String.valueOf(balancePayFee));
                cmUserBalanceRecord.setOrderId(cmOrder.getOrderID());
                cmUserBalanceRecord.setDelFlag("0");
                userMapper.insertUserBalanceRecord(cmUserBalanceRecord);
            }



            /**保存子订单后更新主订单信息*/
            cmOrder.setShopOrderIDs("");//子订单ID:1000,1002

            /**保存订单商品*/
            for(CmOrderProduct product:orderProductList){//循环插入数据
                product.setOrderID(orderID);
                product.setOrderNo(orderNo);
                //保存订单商品数据
                organizeProductsMapper.insert(product);
            }

            String shopOrderIDs = "";//收集子订单信息
            /**保存子订单*/
            Iterator<Map.Entry<Integer, String>> iterator = shopIDs.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry<Integer, String> shopOrder = iterator.next();
                Integer shopID = shopOrder.getKey();
                String note = shopOrder.getValue();


                /**统计子订单金额信息*/
                CmOrderProduct cmOrderProduct = new CmOrderProduct();
                cmOrderProduct.setOrderID(cmOrder.getOrderID());
                cmOrderProduct.setShopID(shopID);
                //查询属于该供应商的所有订单商品信息
                List<CmOrderProduct> cmOrderProductList = orderProductMapper.findList(cmOrderProduct);
                BigDecimal totalAmount = new BigDecimal(0); // 订单总金额
                BigDecimal productAmount = new BigDecimal(0); //商品总金额
                BigDecimal needPayAmount = new BigDecimal(0);//需要支付金额
                BigDecimal preferential = new BigDecimal(0);// 优惠金额
                BigDecimal fee = new BigDecimal(0);//运费
                BigDecimal brokerage = new BigDecimal(0);//佣金
                Double shopProductAmount = 0D;//商品费
                Double shopTaxFee = 0D; //付给供应商税费
                Integer itemCount = 0;//总数
                Integer buyNum = 0;//总购买数
                Integer presentNum = 0;//总赠送数
                //计算子订单信息
                for (CmOrderProduct orderProduct : cmOrderProductList) {
                    productAmount = MathUtil.add(productAmount, orderProduct.getTotalAmount());//商品总金额 包括赠品
                    totalAmount = MathUtil.add(totalAmount, orderProduct.getTotalFee());//订单总金额 包括税费
                    needPayAmount = MathUtil.add(needPayAmount, orderProduct.getShouldPayFee()); //应付金额
                    buyNum += orderProduct.getNum();//总购买数
                    itemCount += (orderProduct.getNum() + orderProduct.getPresentNum());
                    preferential = MathUtil.add(preferential, orderProduct.getPreferential());
                    brokerage = MathUtil.add(brokerage, orderProduct.getCmFee());
                    shopProductAmount += (orderProduct.getShopProductAmount() == null ? 0D : orderProduct.getShopProductAmount());
                }

                //初始化子订单信息
                CmShopOrder cmShopOrder = new CmShopOrder();
                cmShopOrder.setClubID(clubID);
                cmShopOrder.setSpID(null);
                cmShopOrder.setMainSpID(null);
                cmShopOrder.setOrderID(orderID);
                cmShopOrder.setOrderNo(orderNo);
                cmShopOrder.setUserID(userId);
                cmShopOrder.setNote(note);
                cmShopOrder.setOrderTime(dateStr);
                cmShopOrder.setDiscountFee(0d);
                cmShopOrder.setCanRefundFlag(1);
                cmShopOrder.setCanRefundAmount(needPayAmount.doubleValue());
                cmShopOrder.setAccountAmount(0d);
                cmShopOrder.setBrokerage(brokerage.doubleValue());//佣金 采美应收
                cmShopOrder.setBuyStatus("1");
                cmShopOrder.setPresentNum(0);
                cmShopOrder.setUseBeanFlag(0);
                cmShopOrder.setUseBeanAmount(0);
                cmShopOrder.setUseBalanceFlag(0);
                cmShopOrder.setRefundStatus(0);
                cmShopOrder.setRefundsAmount(0d);
                cmShopOrder.setPayStatus("1");
                cmShopOrder.setSendOutStatus("1");
                cmShopOrder.setPayFlag("0");
                cmShopOrder.setOrderStatusFlag("1");
                cmShopOrder.setDelFlag("0");
                cmShopOrder.setItemCount(buyNum);//购买商品数
                cmShopOrder.setPresentNum(0);//赠送数
                cmShopOrder.setShopID(shopID);
                cmShopOrder.setOrderType(1);
                cmShopOrder.setOrderSubmitType(cmOrder.getOrderSubmitType());
                cmShopOrder.setTotalAmount(totalAmount.doubleValue());
                cmShopOrder.setProductAmount(productAmount.doubleValue());
                cmShopOrder.setNeedPayAmount(needPayAmount.doubleValue());
                cmShopOrder.setPreferential(preferential.doubleValue());
                cmShopOrder.setShopProductAmount(shopProductAmount);
                cmShopOrder.setShopPostFee(0D);//付给供应商运费
                cmShopOrder.setShopTaxFee(0D);//付给供应商税费
                cmShopOrder.setPayedShopAmount(0D);//已付款金额
                cmShopOrder.setShopOtherFee(0D);//付第三方
                cmShopOrder.setSplitFlag("0");

                List<CmShopOrder> cmShopOrders1 = cmShopOrderMapper.findMaxShopOrderNo(cmOrder.getOrderID());
                String shopOrderNo = ""; //子订单编号
                if (!CollectionUtils.isEmpty(cmShopOrders1)) {
                    shopOrderNo = cmShopOrders1.get(0).getShopOrderNo();
                    cmShopOrder.setShopOrderNo(NoUtils.getCmShopOrderNo(cmOrder.getOrderNo(), Integer.parseInt(shopOrderNo.substring(shopOrderNo.length() - 2, shopOrderNo.length())) + 1));
                } else {
                    cmShopOrder.setShopOrderNo(NoUtils.getCmShopOrderNo(cmOrder.getOrderNo(), 1));
                }
                //保存子订单信息到数据库
                cmShopOrderMapper.insert(cmShopOrder);
                shopOrderIDs += ((shopOrderIDs.equals("") ? "" : ",") + cmShopOrder.getShopOrderID());
                for (CmOrderProduct orderProduct : cmOrderProductList) {
                    orderProduct.setShopOrderID(cmShopOrder.getShopOrderID());
                    orderProduct.setShopOrderNo(cmShopOrder.getShopOrderNo());
                    orderProductMapper.update(orderProduct);
                }
            }

            //更新主订单信息
            cmOrder.setShopOrderIDs(shopOrderIDs);
            orderMapper.update(cmOrder);

            //地址信息

            //发票信息

            //获取用户最新余额，判断是否存在

            //提交订单完成后清理购物车(用户ID,商品ID)
            if (StringUtils.equals("1", cartType)) {//购物车提交后清理购物车

            }
        }

        return wxJsonModel.success();
    }


    @Override
    public Map<String, Object> logistics(Integer orderID) {
        Map<String, Object> map = new HashMap();
        //物流商品信息
        CmOrder order = orderMapper.findOrder(orderID);
        String[] ShopOrderIDs = {};
        if (order.getShopOrderIDs().contains(",")) {
            ShopOrderIDs = order.getShopOrderIDs().split(",");
        } else {
            ShopOrderIDs = new String[]{order.getShopOrderIDs()};
        }
        List<CmShopOrder> shopOrderList = orderMapper.findAllShopOrder(ShopOrderIDs);
        for (CmShopOrder shopOrder : shopOrderList) {
            List<CmLogisticsRecord> logisticsRecordList = orderMapper.findLogisticsRecord(shopOrder.getShopOrderID());
            for (CmLogisticsRecord logisticsRecord : logisticsRecordList) {
                logisticsRecord.setImage(AppUtils.getImageURL("product", logisticsRecord.getImage(), 0, domain));
            }
            shopOrder.setCmLogisticsRecords(logisticsRecordList);
            shopOrder.setShopLogo(AppUtils.getImageURL("shopLogo", shopOrder.getShopLogo(), 0, domain));
        }
        //物流信息
        List<LogisticsInfo> logisticsInfoList = orderMapper.findLogisticsInfo(orderID);
        for (LogisticsInfo logisticsInfo : logisticsInfoList) {
            List<Router> routers = JSONArray.parseArray(logisticsInfo.getInfo(), Router.class);
            logisticsInfo.setRouters(routers);
        }
        map.put("logisticsInfoList", logisticsInfoList);
        map.put("shopOrderList", shopOrderList);
        return map;
    }

    @Override
    public List<CmOrder> myOrder(Integer userID, Integer orderState, Integer organizeID) {
        List<CmOrder> orderList = orderMapper.findAllOrder(userID, orderState, organizeID);
        for (CmOrder order : orderList) {
            String[] ShopOrderIDs = {};
            if (order.getShopOrderIDs().contains(",")) {
                ShopOrderIDs = order.getShopOrderIDs().split(",");
            } else {
                ShopOrderIDs = new String[]{order.getShopOrderIDs()};
            }
            List<CmShopOrder> shopOrderList = orderMapper.findAllShopOrder(ShopOrderIDs);
            for (CmShopOrder shopOrder : shopOrderList) {
                List<CmOrderProduct> orderProductList = orderMapper.findOrderProduct(shopOrder.getShopOrderID());
                for (CmOrderProduct orderProduct : orderProductList) {
                    orderProduct.setProductImage(AppUtils.getImageURL("product", orderProduct.getProductImage(), 0, domain));
                }
                shopOrder.setCmOrderProducts(orderProductList);
                shopOrder.setShopLogo(AppUtils.getImageURL("shopLogo", shopOrder.getShopLogo(), 0, domain));
            }
            order.setShopOrderList(shopOrderList);
        }
        return orderList;
    }

    @Override
    public List<CmOrganizeProducts> oftenPurchase(Integer userID, Integer organizeID) {
        List<CmOrganizeProducts> organizeProductsList = orderMapper.oftenPurchase(userID, organizeID);
        for (CmOrganizeProducts organizeProducts : organizeProductsList) {
            organizeProducts.setMainImage(AppUtils.getImageURL("product", organizeProducts.getMainImage(), 0, domain));
        }
        return organizeProductsList;
    }
}
